@@ -3,6 +3,8 @@ from __future__ import annotations
 import contextlib
 import logging
 import os
+import warnings
+from functools import partial
 
 import click
 from rich.console import Console, Group
@@ -15,9 +17,16 @@ from typing_extensions import Callable
 
 from patchwork.managed_files import HOME_FOLDER, LOG_FILE
 
+# Create a global console object
 console = Console()
-# default noop logger
+
+# Add TRACE level to logging
+logging.TRACE = logging.DEBUG - 1
+logging.addLevelName(logging.TRACE, "TRACE")
 logger = logging.getLogger("patched")
+logger.trace = partial(logger.log, logging.TRACE)
+
+# default noop logger
 __noop = logging.NullHandler()
 logger.addHandler(__noop)
 
@@ -119,7 +128,7 @@ class TerminalHandler(RichHandler):
         self.__live.refresh()
 
     def __get_filter(self, log_level: str) -> Callable[[logging.LogRecord], bool]:
-        log_level = logging.getLevelName(log_level)
+        log_level = logging.TRACE if log_level == "TRACE" else logging.getLevelName(log_level)
 
         def inner(record: logging.LogRecord) -> bool:
             return record.levelno >= log_level
@@ -130,6 +139,7 @@ class TerminalHandler(RichHandler):
 def init_cli_logger(log_level: str) -> logging.Logger:
     global logger, __noop
 
+    warnings.simplefilter("ignore")
     logger.removeHandler(__noop)
 
     if not os.path.exists(HOME_FOLDER):  # Check if HOME_FOLDER exists at this point
@@ -148,5 +158,5 @@ def init_cli_logger(log_level: str) -> logging.Logger:
     setattr(logger, "register_progress_bar", th.register_progress_bar)
     setattr(logger, "deregister_progress_bar", th.deregister_progress_bar)
     setattr(logger, "freeze", th.freeze)
-    logger.setLevel(logging.DEBUG)
+    logger.setLevel(logging.TRACE)
     return logger
