@@ -2,8 +2,7 @@ import os
 import subprocess
 from pathlib import Path
 
-from patchwork.logger import logger
-from patchwork.step import Step
+from patchwork.step import Step, StepStatus
 
 FOLDER_PATH = "folder_path"
 
@@ -12,8 +11,7 @@ class CallCode2Prompt(Step):
     required_keys = {FOLDER_PATH}
 
     def __init__(self, inputs: dict):
-        logger.info(f"Run started {self.__class__.__name__}")
-
+        super().__init__(inputs)
         if not all(key in inputs.keys() for key in self.required_keys):
             raise ValueError(f'Missing required data: "{self.required_keys}"')
 
@@ -28,9 +26,6 @@ class CallCode2Prompt(Step):
             with open(self.code_file_path, "a") as file:
                 pass  # No need to write anything, just create the file if it doesn't exist
 
-        # Prepare for data extraction
-        self.extracted_data = []
-
     def run(self) -> dict:
         cmd = [
             "code2prompt",
@@ -54,13 +49,13 @@ class CallCode2Prompt(Step):
             with open(self.code_file_path, "r") as file:
                 file_content = file.read()
         except FileNotFoundError:
-            logger.info(f"Unable to find file: {self.code_file_path}")
+            self.set_status(StepStatus.FAILED, f"Unable to find file: {self.code_file_path}")
+            return dict(files_to_patch=[])
 
         lines = file_content.splitlines(keepends=True)
 
-        self.extracted_data.append(
-            dict(uri=self.code_file_path, startLine=0, endLine=len(lines), fullContent=prompt_content_md)
+        return dict(
+            files_to_patch=[
+                dict(uri=self.code_file_path, startLine=0, endLine=len(lines), fullContent=prompt_content_md)
+            ]
         )
-
-        logger.info(f"Run completed {self.__class__.__name__}")
-        return dict(files_to_patch=self.extracted_data)
